#include "glview.h"


GLView::GLView(Scene *scene,Camera * camera,Controler *controler )
{
    this->camera = camera;
    this->scene = scene;
    this->controler = controler;
}

QSize GLView::minimumSizeHint() const
{
    return QSize(50, 50);
}

QSize GLView::sizeHint() const
{
    return QSize(600, 400);
}


void GLView::initializeGL ( ) {
    glClearColor(0.0, 0.0, 0.0, 0.0);
    glEnable(GL_DEPTH_TEST);


    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    static GLfloat lightPosition[4] = { 0.0, 0.0, 4.0, 1.0 };
    glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);

    //Shader Setup
    initShader();
    shader->bind();
}

void GLView::initShader()
{
    QGLShader *vertex = new QGLShader(QGLShader::Vertex);
    if(!vertex->compileSourceFile(QLatin1String(":/phong.vert")))
        qCritical()<< "Vertex Shader failed"<< vertex->log();

    QGLShader *fragment = new QGLShader(QGLShader::Fragment);
    if(!fragment->compileSourceFile(QLatin1String(":/phong.frag")))
        qCritical()<< "Fragment Shader failed"<< fragment->log();

    shader = new QGLShaderProgram(this);
    shader->addShader(vertex);
    shader->addShader(fragment);
    if(!shader->link())
        qCritical()<< "Linking  failed"<<shader->log();
}

void GLView::home(){
    camera->home();
    updateGL();
}



void GLView::paintGL ()
{
  //  QGLFramebufferObjectFormat format;
  //  format.setSamples();
  //  format.setAttachment(QGLFramebufferObject::);

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glEnable(GL_DEPTH_TEST);

    glViewport(0,0,this->width(),this->height());

    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity ();
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();


    shader->setUniformValue("shaded",true);


    //set Projection and Camera Rotation
    camera->setupCamera(aspect);

    //draw Scene
    scene->draw();

    if(isActive){
        glDisable(GL_DEPTH_TEST);
        glMatrixMode (GL_MODELVIEW);
        glLoadIdentity ();
        glMatrixMode (GL_PROJECTION);
        glLoadIdentity ();
        shader->setUniformValue("shaded",false);
        glLineWidth(10);
        GLfloat color[] = {1.0,1.0,0.0};
        glMaterialfv(GL_FRONT,GL_AMBIENT,color);
        glMaterialfv(GL_FRONT,GL_DIFFUSE,color);
        glMaterialfv(GL_FRONT,GL_SPECULAR,color);
        glMaterialf(GL_FRONT,GL_SHININESS,128);

        glBegin (GL_LINE_LOOP);
        glVertex3i (-1, -1, 0);
        glVertex3i (1, -1, 0);
        glVertex3i (1, 1, 0);
        glVertex3i (-1, 1, 0);
        glEnd ();
    }


}
void GLView::resizeGL(int width , int height )
{
    aspect = 1.0*width/height;
}



void GLView::setAcive(bool active)
{
    this->isActive = active;
    updateGL();
}

Camera *GLView::getCamera()
{
    return this->camera;
}


void GLView::mousePressEvent(QMouseEvent *event )
{
    controler->mousePressed(this,event);
}

void GLView::mouseMoveEvent(QMouseEvent *event )
{
     controler->mouseMoveEvent(this,event);
}

void GLView::wheelEvent(QWheelEvent *event )
{
    controler->wheelEvent(this,event);
}

